#!/usr/bin/env python3
from misc import evpyprint
newline=evpyprint.newline
nonewline=evpyprint.nonewline
import os, sys, re
#import math
import numpy
import scipy.stats
import projtls
import argparse
from misc import plural

raw_input = input
#import formattable
# 2dsinth=lam
# 1/d = 2*sinth/lam

#        arg = 0.5*lambdamean/reso
#        if(arg.le.1.0) then
#          theta = asin(arg)
#        endif
#        sin(theta) = 0.5*lambdamean/reso
#        reso = 0.5*lambdamean/sin(theta)
#        sinth = arg
#        sinthl = 
#        sinthl=float(0.5*absvec( matvec(rmat,v) ))
#        sinth = wave*sinthl
#        reso = 0.5*wave/(wave*sinthl) = 0.5/sinthl
_prog_usage = '%(prog)s [options] fcffile'
_prog_version = 'anafcf Version 1.3'
_prog_description = 'analysis of fcf reflection file'
_prog_author = 'Toine Schreurs'

DEGREES = numpy.pi/180.0
INCH = 2.54
PAGEWIDTH = 11.5
PAGEHEIGHT = 8.3
GAP = 0.01
EXTRAUPPER = 1.0e-6
SUBWINDOWWIDTH = 0.18

def absvec(v):
    """Determine length of single 3D vector or each out of an array of vectors"""
    if v.shape[-1]!=3:
        raise ValueError("Invalid shape for v in vectorlength")
    return numpy.sqrt(v[...,0]*v[...,0]+v[...,1]*v[...,1]+v[...,2]*v[...,2])

matvec = numpy.inner
invert = numpy.linalg.inv
cos = numpy.cos
sin = numpy.sin
 
def alias(s):
# for more information: search for mathtext
    aliases = { 
        'iobs'            : r'$I_{obs}$',
        'iabs'            : r'$I_{abs}$',
        'icalc'           : r'$I_{calc}$',
        'sigma'           : r'$\sigma$',
        'delta/sigma'     : r'$\Delta/\sigma$',
        'iobs/sigma'      : r'$I_{obs}/\sigma$',
        'ioversig'        : r'$I_{obs}/\sigma$',
        'delta'           : r'$\Delta$',
        'delsig'          : r'$\Delta/\sigma$',
        'delsig2'         : r'$\Delta/\sigma^{2}$',
        'del2sig2'        : r'$\Delta^{2}/\sigma^{2}$',
        'delta/sigma^2'   : r'$\Delta/\sigma^{2}$',
        'delta^2/sigma^2' : r'$\Delta^{2}/\sigma^{2}$',
        'iobs/icalc'      : r'$I_{obs}/I_{calc}$',
        'sinthl'          : r'$\sin(\theta)/\lambda$',
        'sinthl2'         : r'$\sin^{2}(\theta)/\lambda^{2}$',
        'logiobs'         : r'$\log(I_{obs})$',
        'lniobsiabs'      : r'$\ln(I_{obs}/I_{abs})$',
        'sqrtiobs'        : r'$\sqrt{I_{obs}}$',
        'logicalc'        : r'$\log(I_{calc})$',
        'sqrticalc'       : r'$\sqrt{I_{calc}}$',
        'prob'            : r'$prob$',
        'S'               : r'$\sqrt{\Delta^{2}/\sigma^{2}}$',
        'theta'           : r'$\theta$',
        'lambda'          : r'$\lambda$'
        }
    if s in aliases:
        return aliases[s]
    else:
        return s
    
def hklkey(hkl):

    def safe_int(obj):
        try:
            retval = int(obj)
        except (ValueError,TypeError) as diag:
            retval = str(diag)
        return retval

    v=[]
    for x in hkl:
        n = safe_int(x)
        if isinstance(n,int):
            v.append(n)
        else:
            newline(n)
            return None
    hkl = '%d %d %d'%tuple(v) 
    return hkl


def celldim(cel):
    a,b,c = splitmat(cel)
    len_a=absvec(a)
    a=a/len_a
    len_b=absvec(b)
    b=b/len_b
    len_c=absvec(c)
    c=c/len_c
    if len_a and len_b and len_c:
        alpha=numpy.arccos(numpy.add.reduce(b*c))
        beta= numpy.arccos(numpy.add.reduce(c*a))
        gamma=numpy.arccos(numpy.add.reduce(a*b))
        return (len_a,len_b,len_c,alpha,beta,gamma)
    else:
        raise error("Zero axis length")

def splitmat(m):
    """Split a matrix into its 3 columns"""
    return m[:,0],m[:,1],m[:,2]

def reccel(cel):
    """Calculate reciprocal cell parameters from direct cell parameters in a 6-tuple"""
    a,b,c,alpha,beta,gamma=cel[0],cel[1],cel[2],cel[3],cel[4],cel[5]
    alphas=numpy.arccos((cos(beta)*cos(gamma)-cos(alpha))/(sin(beta)*sin(gamma)))
    betas=numpy.arccos((cos(alpha)*cos(gamma)-cos(beta))/(sin(alpha)*sin(gamma)))
    gammas=numpy.arccos((cos(alpha)*cos(beta)-cos(gamma))/(sin(alpha)*sin(beta)))
    astar=1./(a*sin(beta)*sin(gammas))
    bstar=1./(b*sin(alpha)*sin(gammas))
    cstar=1./(c*sin(alpha)*sin(betas))
    return astar,bstar,cstar,alphas,betas,gammas

def daxdir(cel):
    """Make standard-orientation direct cell matrix from cell parameters
    """
    dm=numpy.zeros((3,3),'d')

    dm[0,0] = cel[0]
    dm[0,1] = 0.0
    dm[0,2] = 0.0

    dm[1,0] = cel[1]*cos(cel[5])
    dm[1,1] = cel[1]*sin(cel[5])
    dm[1,2] = 0.0

    dm[2,0] = cel[2]*cos(cel[4])
    dm[2,1] = (cel[1]*cel[2]*cos(cel[3])-dm[1,0]*dm[2,0]) /dm[1,1]
    dm[2,2] = cel[2]**2 - dm[2,0]**2 - dm[2,1]**2

    if dm[2,2] < 0.01:
        printcell('error:',cel)
        raise UserWarning('daxdir error')
    else:
        dm[2,2] = numpy.sqrt(dm[2,2])

#    return numpy.transpose(dm)
    return dm

def printcell(s,cell):
    newline('%s: %f %f %f %f %f %f'%(s,cell[0],cell[1],cell[2],cell[3]/DEGREES,cell[4]/DEGREES,cell[5]/DEGREES))

class Reflrec(object):
    '''reflection record'''

    def __init__(self,items):
        for value,name,func in items:
            if len(value) and not value.isspace():
                setattr(self,name,func(value))
        if hasattr(self,'h') and hasattr(self,'k') and hasattr(self,'l'):
            self.hkl = hklkey([self.h, self.k, self.l])
#            self.hkl='%d %d %d'%(self.h,self.k,self.l) 
        if hasattr(self,'iobs'):
            self.logiobs = numpy.log10(max( 1.0,self.iobs))
            self.sqrtiobs = numpy.sqrt(max( 1.0,self.iobs))
        if hasattr(self,'icalc'):
            self.logicalc = numpy.log10(max( 1.0,self.icalc))
            self.sqrticalc = numpy.sqrt(max( 1.0,self.icalc))
        if hasattr(self,'icalc') and hasattr(self,'iobs'):
            self.delta = self.iobs - self.icalc
        if hasattr(self,'icalc') and hasattr(self,'iobs'):
            self.q = self.iobs/max(0.0001,self.icalc)
        if hasattr(self,'sigma') and hasattr(self,'iobs') and hasattr(self,'icalc'):
            p = (max(self.iobs,0.0) + 2*self.icalc)/3.0
            self.wsig = numpy.sqrt( self.sigma**2 + (cifdict['weighta']*p)**2 + cifdict['weightb']*p)
        if not args.ignorefcfsigma:
            if hasattr(self,'refsig'):
                self.sigma = self.refsig
            elif hasattr(self,'wsig'):
                self.sigma = self.wsig

        if hasattr(self,'sigma') and hasattr(self,'iobs'):
            if self.sigma < 1.0e-6:
                newline(self)
            else:
                self.ioversig =self.iobs/self.sigma
        if hasattr(self,'delta') and hasattr(self,'sigma'):
            self.delsig = self.delta/self.sigma
            self.delsig2 = self.delta/(self.sigma**2)
            self.del2sig2 = (self.delta**2)/(self.sigma**2)

    def legenda(self):
        # return a list of tuples (name,value) in a preferred order
        preferred = ['hkl','iobs','sigma','ioversig','icalc','delta','d','theta','sinthl','sinthl2']
        ignore = []
        if 'hkl' in self.__dict__ :
            ignore = ['h','k','l']
#        ignore.append('theta')
        items = []
        for item in preferred:
            if hasattr(self,item):
                items.append(item)
        for item in sorted(self.__dict__):
            if item not in items and item not in ignore:
                items.append(item)
        s=[]
        for item in items:
            a=getattr(self,item)
            if isinstance(a,float):
                if item in ['theta']:
                    a = a/DEGREES
                s.append((item,'%.2f'%a))
            else:
                s.append((item,str(a)))
        return s

    def __str__(self):
        s = []
        for item in self.legenda():
            s.append('%s=%s'%(item))
        return ' '.join(s)

    def __cmp__(self,other):
        return cmp(self.intensity,other.intensity)

def operator():
    line = raw_input('type continue ')
    if line == 'q':
        raise

'''
r"calc w=1/[\s^2^(Fo^2^)+(0.0182P)^2^+0.5088P] where P=(Fo^2^+2Fc^2^)/3"
leading whitespace and trailing whitespace
r"\s*calc w=1/[\s^2^(Fo^2^)+(0.0182P)^2^+0.5088P] where P=(Fo^2^+2Fc^2^)/3\s*$"
escape special characters / \ [ ( ^ +
r"\s*calc w=1/\[\\s\^2\^\(Fo\^2\^\)\+\(0.0182P\)\^2\^\+0.5088P\] where P=\(Fo\^2\^\+2Fc\^2\^\)/3"
grouping
r"\s*calc w=1/\[\\s\^2\^\(Fo\^2\^\)\+\((0.0182)P\)\^2\^\+(0.5088)P\] where P=\(Fo\^2\^\+2Fc\^2\^\)/3"
floats
r"\s*calc w=1/\[\\s\^2\^\(Fo\^2\^\)\+\((\d+\.\d+)P\)\^2\^\+(\d+\.\d+)P\] where P=\(Fo\^2\^\+2Fc\^2\^\)/3"

 w=1/[\s^2^(Fo^2^)+(12.34P)^2^+56.78P]
 w=1/[\s^2^(Fo^2^)+(12.34P)^2^]
 w=1/[\s^2^(Fo^2^)+56.78P]

'''

# Note, the most rigid test first....
CIFDescription=(
    (re.compile(r".*w=1/\[.*\+\((\d+\.\d+)P\).*\+(\d+\.\d+)P\]"),  
     ((1, "f","weighta"),(2, "f","weightb"))),
    (re.compile(r".*w=1/\[.*\+\((\d+\.\d+)P\).*\]"),
     ((1, "f","weighta"),)),
    (re.compile(r".*w=1/\[.*\+(\d+\.\d+)P\]"),
     ((1, "f","weightb"),)),
    (re.compile(r"\s*_audit_creation_method\s+(\S+)\s*$"),
     ((1, "s","method"),)),
    (re.compile(r"data_(.+)$"),
     ((1,"s","data"),)),
    (re.compile(r"\s*_diffrn_radiation_wavelength\s+(\d+\.\d+)\s*$"),
     ((1,"f","wavelength"),)),
    (re.compile(r"\s*_refine_ls_number_parameters\s+(\d+)\s*$"),
     ((1,"i","lsparameters"),)),
    )

class PLOTrecord(object):
    def __init__(self,rec=None):
        self.rec=rec

    def Defaults(self):
        self.rec = projtls.Record()
        self.type = None
        self.Redline = False
        self.label = None

    def scatter(self, hor, ver, Redline=False, XY=False, label=None):
        self.Defaults()
        self.type = 'scatter'
        self.hor = hor[0]
        self.horlimit = hor[1]
        self.horred = hor[2]
        self.xtext = hor[3]
        self.ver = ver[0]
        self.verlimit = ver[1]
        self.verred = ver[2]
        self.ytext = ver[3]
        self.Redline = Redline
        self.label = label
        if XY:
            self.horred[0] = min(hor[2][0],ver[2][0])
            self.verred[0] = self.horred[0]
            self.horred[1] = max(hor[2][1],ver[2][1])
            self.verred[1] = self.horred[1]

    def bar(self, bd):
        # bins averages count hor name
        self.Defaults()
        self.type = 'bar'
        hor = bd[0]
        ver = bd[1]
        self.count = bd[2]
        self.hor = hor
        self.ver = ver
        self.horlimit = numpy.array( [0.0, hor.size] )
        self.verlimit = numpy.array( [ min(0.0,numpy.amin(ver)), numpy.amax(ver)] )
        self.xtext = bd[3]
        self.ytext = bd[4]
        #newline(self.xtext,self.horlimit)
        #newline(self.ytext,self.verlimit)

    def point(self, bd):
        # bins averages count hor name
        self.Defaults()
        self.type = 'point'
        hor = bd[0]
        ver = bd[1]
        self.hor = hor
        self.ver = ver
        self.count = bd[2]
        self.horlimit = numpy.array( [0.0, numpy.amax(hor)] )
        self.verlimit = numpy.array( [ min(0.0,numpy.amin(ver)), numpy.amax(ver)] )
        self.xtext = bd[3]
        self.ytext = bd[4]
        #newline(self.xtext,self.horlimit)
        #newline(self.ytext,self.verlimit)

class CIFrecord:
    def __init__(self,rec = None):
        self.rec = rec

    def Defaults(self):
        self.rec = projtls.Record()

    def setFilename(self, fn):
        self.rec.filename = fn

    def Read(self,filename=''):
        f=open(filename,'r')
        txt=f.readlines()
        f.close()
        self.Defaults()
        self.setFilename(filename)
        self.rec=projtls.ReScanText(txt,CIFDescription,into=self.rec,allowunknown=2)

    def __repr__(self):
        return repr(self.rec)

def ReadCIF(filename):
    ciffilename = filename + '.cif'
    if not os.path.exists(ciffilename):
        ciffilename = filename + '_lsm' + '.cif'
    if os.path.exists(ciffilename):
        newline('reading %s'%ciffilename)
        cif = CIFrecord()
        cif.Read(ciffilename)
        return cif
    else:
        return None

def bracefloat(s):
    return float(s.partition('(')[0])

def ReadFCF(filename,rmat,wavelength=0.0):
    # create empty reflection list
    refls=[]
    # create a list of requested items
    requests = []
    newline('reading %s'%filename)
    f = open(filename,'r')
    WithinLoop = False
    WithinReflLoop = False
    cell = []
    nignore = 0
    for line in f:
        line = line.strip()
        n = line.find('#')
        if n >= 0:
            line = line[:n]
        if len(line) == 0:
            WithinLoop = False
            WithinReflLoop = False
        elif line[:5] == 'loop_':
            WithinLoop = True
        elif line[:6] == '_cell_':
            items = line.split()
            if items[0][6:12] == 'length':
                cell.append(bracefloat(items[1]))
            elif items[0][6:11] == 'angle':
                cell.append(bracefloat(items[1])*DEGREES)
            if len(cell) == 6:
                dmat = daxdir(cell)
                rmat = invert(dmat)
                # newline(rmat)
        elif line[:7] == '_refln_' :
            WithinReflLoop = WithinLoop
            if line[7:] == 'index_h':
                requests.append(('h',int))
            elif line[7:] == 'index_k':
                requests.append(('k',int))
            elif line[7:] == 'index_l':
                requests.append(('l',int))
            elif line[7:] == 'F_squared_calc':
                requests.append(('icalc',float))
            elif line[7:] == 'F_squared_meas':
                requests.append(('iobs',float))
            elif line[7:] == 'F_squared_sigma':
                requests.append(('sigma',float))
            elif line[7:] == 'observed_status':
                requests.append(('status',str))
            elif line[7:] == 'XD_refine_code':
                requests.append(('status',int))
            else:
                newline('  ignoring %s'%line)
                requests.append((None,None))
        elif line[:7] ==  '_shelx_' :
            WithinReflLoop = WithinLoop
            if WithinReflLoop:
                if line[7:] == 'refinement_sigma':
                    requests.append(('refsig',float))
                else:
                    newline('  ignoring %s'%line)
#                requests.append((None,None))
        elif WithinReflLoop:
            if args.verbose > 1 and len(refls) == 0:
                newline(requests)
            items = line.split()
            fields = []
            for item,request in zip(items, requests):
                if request[0] != None:
                    fields.append( ( item, request[0], request[1]) )
            new = Reflrec(fields)
            v=numpy.array([new.h,new.k,new.l])
            new.sinthl=float(0.5*absvec( matvec(rmat,v) ))
            new.d=0.5/new.sinthl
            if wavelength > 0.01:
                new.theta=numpy.arcsin(wavelength*new.sinthl)
            new.sinthl2 = new.sinthl**2
            refls.append(new)
        
    f.close
    if args.verbose > 0:
        printcell('cell from input',cell)
        printcell('reciprocal',reccel(cell))
        if len(refls) > 0:
            if not args.ignorefcfsigma:
                if hasattr(refls[0],'refsig'):
                    newline('using refined sigma')
                else:
                    newline('using weight corrected sigma')
    dmat = daxdir(cell)
    rmat = invert(dmat)

    return rmat,refls

def ReadHKLF4(filename,rmat,wavelength=None):
    # create empty reflection list
    refls=[]
    # create a list of requested items
    requests = [ ('h',int), ('k',int), ('l',int), ('iobs',float), ('sigma',float) ]
    f = open(filename,'r')
    newline('reading %s'%filename)
    for line in f:
        if len(line) <= 1:
            break
        items = [ line[0:4], line[4:8], line[8:12], line[12:20], line[20:28] ]
        fields = []
        for item,request in zip(items,requests):
            fields.append( (item,request[0],request[1]) )
        new = Reflrec(fields)
        if new.h == 0 and new.k == 0 and new.l == 0:
            break
        v=numpy.array([new.h,new.k,new.l])
        new.sinthl=float(0.5*absvec( matvec(rmat,v) ))
        new.d=0.5/new.sinthl
        if wavelength > 0.01:
            new.theta=numpy.arcsin(wavelength*new.sinthl)
        new.sinthl2 = new.sinthl**2

        refls.append(new)
    f.close
    newline('  %s'%plural.reflections(refls))
    return rmat,refls

def ReadRefls(filename,rmat,wavelength=None):
    funcs={
        '.hkl':ReadHKLF4,
        '.hklv':ReadHKLF4,
        '.sad':ReadHKLF4,
        '.hkp':ReadHKLF4,
#        '.hk6':ReadHK6,
#        '.ram':ReadHK6,
        '.fcf':ReadFCF,
        '.fco':ReadFCF
        }
    root,ft=os.path.splitext(filename)
    try:
        newline('reflectionfile=%s'%filename)
        if rmat is not None:
            newline('rmat=',rmat)
        newline('wavelength=%f'%wavelength)
        rmat,refls = funcs[ft](filename,rmat=rmat,wavelength=wavelength)
    except KeyError:
        newline('unknown filetype %s in %s'%(ft,filename))
        newline('possible types are: %s'%(' '.join(funcs)))
        raise IOError('bad filetype')

    return rmat,refls

# def gen117(n):
#     def gen116(p):
#         p0 = -0.322232431088
#         p1 = -1.0
#         p2 = -0.342242088547
#         p3 = -0.204231210245E-1
#         p4 = -0.453642210148E-4
#         q0 = 0.993484626060E-1
#         q1 = 0.588581570495
#         q2 = 0.531103462366
#         q3 = 0.103537752850
#         q4 = 0.38560700634E-2
#         if p == 0.5:
#             ppf = .0
#         else:
#             r = p
#             if p > 0.5:
#                 r = 1.0-r
#             t = numpy.sqrt(-2.0*numpy.log(r))
#             anum = ((((t*p4+p3)*t+p2)*t+p1)*t+p0)
#             aden = ((((t*q4+q3)*t+q2)*t+q1)*t+q0)
#             ppf = t + (anum/aden)
#             if p < 0.5:
#                 ppf = -ppf
#         return ppf

#     a = numpy.zeros(n)
#     f = 0.5**(1.0/n)
#     a[0] = gen116( 1.0-f )
#     a[n-1] = gen116( f ) 
#     if n%2 != 0:
#         nhalf = n//2 
#         a[nhalf] = gen116( 0.5 )
#     gam = 0.3175
#     imax = n//2
#     for i in range(1,imax):
#         irev = n - i - 1
#         f =  (i+1-gam)/(n-2.0*gam+1.0)
#         a[i] =  gen116( f ) 
#         a[irev] =  gen116( 1.0-f )
#     return a


def SetWeightsAndWavelength(args,filename,filetype):
    weighta = args.weighta
    weightb = args.weightb
    weightreport = []
    if args.wavelength is not None:
        wavelength = args.wavelength
    else:
        wavelength = 0.0

    if args.lsparameters is not None:
        lsparameters = args.lsparameters
    else:
        lsparameters = 0

    if weighta >= 0.0 and weightb < 0.0:
        weightb = 0.0

    if weightb >= 0.0 and weighta < 0.0:
        weighta = 0.0

    if weighta >= 0.0 and weightb >= 0.0:
        args.weightstatus = 'user supplied weights'
        weightreport.append('%s: a=%f b=%f' % (args.weightstatus,weighta,weightb))

    cif = ReadCIF(filename)
    if cif:
        if args.verbose > 1:
            newline(cif)
        if hasattr(cif.rec,'data'):
            newline('  cif data_%s' % cif.rec.data)
        else:
            newline('WARNING: no cif data field found')
        if weighta < 0.0 and weightb < 0:
            if not hasattr(cif.rec,'weighta') and not hasattr(cif.rec,'weightb'):
                weightreport.append('  no weights found in ciffile')
            else:
                if hasattr(cif.rec,'weighta'):
                    weighta = cif.rec.weighta
                    weightreport.append('  Using cif weight a=%f' %weighta)
                else:
                    weightreport.append('  Weight b found in ciffile, but no weight a ==> weight a=0')
                    weighta = 0.0
                if hasattr(cif.rec,'weightb'):
                    weightb = cif.rec.weightb
                    weightreport.append('  Using cif weight b=%f' %weightb)
                else:
                    weightreport.append('  Weight a found in ciffile, but no weight b ==> weight b=0')
                    weightb = 0.0
            if weighta >= 0 and weightb >= 0:
                args.weightstatus = 'cif supplied weights'
                weightreport.append('%s: a=%f b=%f' % (args.weightstatus,weighta,weightb))
                
        if hasattr(cif.rec,'wavelength'):
            if args.wavelength is None:
                wavelength = cif.rec.wavelength
                newline('  Using wavelength %f' %wavelength)
            else:
                newline('  Ignoring wavelength %f'%cif.rec.wavelength)
        else:
            newline('  No wavelength found in ciffile')

        if hasattr(cif.rec,'lsparameters'):
            if args.lsparameters is None:
                lsparameters = cif.rec.lsparameters
                newline('  Using lsparameters %d' %lsparameters)
            else:
                newline('  Ignoring lsparameters %d' %cif.rec.lsparameters)
        else:
            newline('  No refine_ls_number_parameters found in ciffile')

    if weighta < 0.0 and weightb < 0:
        if filetype == '.fco':
            weightreport.append('Setting default weights to 0.0')
            weighta = 0.0
            weightb = 0.0
            args.weightstatus = 'fco: zero weights'
            weightreport.append('%s: a=%f b=%f' % (args.weightstatus,weighta,weightb))
        elif filetype == '.fcf':
            weightreport.append('Setting default weights to 0.0')
            weighta = 0.0
            weightb = 0.0
            args.weightstatus = 'fcf: zero weights'
            weightreport.append('%s: a=%f b=%f' % (args.weightstatus,weighta,weightb))
    elif weighta < 0:
        weighta = 0.1
        args.weightstatus = 'default weight a'
        weightreport.append('%s: a=%f b=%f' % (args.weightstatus,weighta,weightb))
    elif weightb < 0:
        weightb = 0.0
        args.weightstatus = 'default weight b'
        weightreport.append('%s: a=%f b=%f' % (args.weightstatus,weighta,weightb))

    cifdict = {
        'weighta':weighta,
        'weightb':weightb,
        'wavelength':wavelength,
        'lsparameters':lsparameters,
        }
    return cifdict,weightreport

def main():
    import matplotlib as mpl
    newline('fname=',mpl.matplotlib_fname())
    import matplotlib.pyplot as plt
    from matplotlib import rc
    import matplotlib.widgets as wid
    #import widgets as wid
    global mode
    global onscreen
    global ndx
    global cifdict

    def getScale():
        if hasattr(refls[0],'icalc'):
            sumA = 0.0
            sumB = 0.0
            for refl in refls:
                sumA += refl.icalc
                sumB += refl.iobs
            newline('sumA = %.2f sumB =%.2f'%(sumA,sumB))
            scale = sumA/sumB
            newline('scale factor for icalc = %f (ignored)'%scale)
        scale = 1.0
        return scale

    def GetBins(var,n,d):
        scr = numpy.array(d)
        scrs = numpy.sort(scr)
        if args.verbose > 1:
            newline('GetBins nbin=%d variable=%s first=%f, last=%f'%(n,var,scrs[0],scrs[-1]))
            newline('len(d) = %d'%(len(d)))
        # make sure the upper limit will be larger than the largest value
        scrs[-1] = scrs[-1] + EXTRAUPPER
        # note, bins are interpreted as upper limits.
        # generate interval limits, one more than the number of bins
        x = numpy.linspace(0, scr.size-1, n+1)
        # the values at positions x contain the bin limits
        # the first point in x will be ignored (it's a lower limit)
        if var == 'sinthl2':
            if args.bins:
                newline('bins from commandline')
                bins = numpy.array(args.bins)
                if args.verbose > 1 or True:
                    newline(bins[-1],)
                bins[-1] = scrs[-1] + EXTRAUPPER 
                if args.verbose > 1 or True:
                    newline('==>',bins[-1])
            else:
                newline('bins linear in sinthl2')
                bins = numpy.linspace(0.0, scrs[-1], n+1) [1:]
        else:
            bins = numpy.zeros(n)
            if args.verbose > 1:
                newline(var)
                newline('x',type(x),type(x[1]))
                newline(x)
                newline('scrs',type(scrs),type(scrs[1]))
                newline(scrs)
                newline(bins)
            for g in range(n):
                bins[g] = scrs[ int(x[g+1]) ]
            #bins = numpy.linspace(0.0, scrs[-1], n+1) [1:]
            
        if args.verbose > 1:
            nonewline('%d bins='%(len(bins)))
            newline(bins)
        inds = numpy.digitize(scr,bins)
        return bins,inds

    def drawhkl():
        global onscreen
        if ndx >= 0:
            if args.verbose > 1:
                newline('active reflection nr = ',ndx)
            pf = plotfuncs[mode]
            if pf.type == 'scatter':
                if 'hkl' in onscreen:
                    L2 = onscreen['hkl']
                    L2.set_xdata(pf.hor[ndx])
                    L2.set_ydata(pf.ver[ndx])
                    L2.set_visible(True)
                else:
                    L2,=ax1.plot(pf.hor[ndx],pf.ver[ndx],'r.',markersize=2*args.dotsize)
                    onscreen['hkl']=L2

            legenda = refls[ndx].legenda()
            if args.verbose > 1:
                newline('legenda:',legenda)
                newline('str:',refls[ndx])

            for pos,txt in zip(legendab,legenda):
                plt.setp( pos, text=txt[1] )
        else:
            for pos in legendab:
                plt.setp(pos,text=r' ')
            if 'hkl' in onscreen:
                L2 = onscreen['hkl']
                L2.set_visible(False)
        plt.draw()

    def CreateTable(bottom,height,labels,columns=1):
        ll= len(labels)
        newax = plt.axes([0.01, bottom, SUBWINDOWWIDTH , height])
        newax.set_navigate(False)
        newax.set_xticks([])
        newax.set_yticks([])
        dy = 1./(ll)
        ys = numpy.linspace(1-0.5*dy, 0.5*dy, ll)
        la = []
        lb = []
        for y,label in zip(ys,labels):
            s = alias(label[0])
            la.append( newax.text(0.05, y, s, verticalalignment='center' ))
            if columns == 2:
                s = label[1]
            else:
                s = ''
            lb.append( newax.text(0.95, y, s, 
                                  horizontalalignment='right',
                                  verticalalignment='center') )
        return (la,lb)

    def on_pick(event):
        """
        If a dot is picked, highlight the reflection, 
        and display reflection information in the table.
        ndx is global (if modes are switched, the ndx reflection has 
        to be highlighted also).
        """
        global ndx
        if args.verbose > 1:
            newline('on pick:',event.ind)
        newline()
        lastndx = ndx
        if len(event.ind) > 0:
            h = plotfuncs[mode].hor
            v = plotfuncs[mode].ver
            refh = event.mouseevent.xdata
            refv = event.mouseevent.ydata
            d = [ (h[i]-refh)**2 + (v[i]-refv)**2 for i in event.ind ]

            ndx = event.ind[ d.index( min(d) ) ]
            if args.verbose > 1:
                newline('refh=%.2f refv=%.2f'%(refh,refv))
                if len(d) > 1:
                    newline(d)
                newline('min(d)=%f ndx=%d'%(min(d),ndx))
            if ndx == lastndx:
                ndx = -1
            drawhkl()
            printhkl()

    def printhkl():
        if ndx >= 0:
            if args.verbose > 0:
                newline('index=%s'%ndx)
            newline(refls[ndx])

    def createButtons(bottom):
        buttons = {}
        height = 0.04
        width = 0.06
        x = 0.01
        buttonax = plt.axes([x, bottom, width, height])
        button = wid.Button(buttonax, 'Quit')
        button.on_clicked(quitbuttonfunc)
        buttons['quit'] = button
        x = x + width
        buttonax = plt.axes([x, bottom, width, height])
        # button = wid.Button(buttonax, 'Save')
        button = wid.Button(buttonax, 'All-%s'%args.savetype)
        button.on_clicked(savebuttonfunc)
        buttons['save'] = button
        x = x + width
        buttonax = plt.axes([x, bottom, width, height])
        button = wid.Button(buttonax, 'HKL')
        button.on_clicked(hklbuttonfunc)
        buttons['hkl'] = button
        bottom = bottom + height + GAP
        return bottom,buttons

    def createRadios(bottom):
        def change_buttons(r, radiusfactor,circlex=0.05,labelx=0.1):
            "Resize and move all radio buttons in `r` collection by fractions `f`"
            for c in r.circles:
                c.set_radius(c.get_radius()*radiusfactor)
                c.center = (circlex,c.center[1])
            for c in r.labels:
#                c.set_fontsize(10)
                old = c.get_position()
                c.set_position((labelx,old[1]))
            rpos = r.ax.get_position().get_points()
            fh = fig.get_figheight()
            fw = fig.get_figwidth()
            rscale = (rpos[:,1].ptp() / rpos[:,0].ptp()) * (fh / fw)
            for c in r.circles:
                c.height /= rscale
        radios = {}
        height = 1.1*len(modes)*args.charheight
        radioax = plt.axes([0.01, bottom, SUBWINDOWWIDTH, height])
        radio = wid.RadioButtons(radioax, modes, active=mode)
        change_buttons(radio,0.6) # shrink your buttons in half
        radio.on_clicked(switchfunc)
        radios['switch'] = radio
        bottom = bottom + height + GAP
        return bottom,radios

    def triggerSwitch(m):
        n = modes.index(m)
        radio = radios['switch']
        for i,p in enumerate(radio.circles):
            if i==n:
                color = radio.activecolor
            else:
                try:
                    color = radio.ax.get_facecolor()
                except AttributeError:
                    color = radio.ax.get_axis_bgcolor()
            p.set_facecolor(color)
        if radio.drawon: radio.ax.figure.canvas.draw()
        switchfunc(modes[n])

    def quitbuttonfunc(event):
        sys.exit()

    def pressrelease(event):
        if event.name == 'key_press_event':
            if event.key=='q':
                quitbuttonfunc('q-press')
            elif event.key == 'down':
                triggerSwitch(modes[(mode+1)%len(modes)])
            elif event.key == 'up':
                triggerSwitch(modes[(mode-1)%len(modes)])
            else:
                newline('ignored %s %s'%(event.name,event.key))
        else:
            newline('ignored %s %s'%(event.name,event.key))

    def ableButtons(state):
        for key,value in buttons.items():
            if key != 'quit':
                value.eventson = state

    def savebuttonfunc(event):
        ableButtons(False)
        lastmode = mode
        tmpl = 'anafcf-%s-%02d.'+args.savetype
        newline(tmpl)
        for button in buttons:
            buttons[button].ax.set_visible(False)

        for i,m in enumerate(modes):
            triggerSwitch(m)
            fname = tmpl%(filename,i+1)
            newline('save to',fname)
            plt.savefig(fname)
        for button in buttons:
            buttons[button].ax.set_visible(True)
        if i != lastmode:
            triggerSwitch(modes[lastmode])
        ableButtons(True)
        newline('%s saved'%plural.nplural(modes,'picture'))

    def hklbuttonfunc(event):
        """
        Wait for h,k,l from input. 
        Highlight available selected reflection.
        The table will be cleared if the reflection is not available.
        """
        global ndx
        ableButtons(False)
        name = raw_input('Enter hkl:')
        items = name.replace(',',' ').split()
        if len(items) != 3:
            newline('expected 3 numbers')
            ndx = -1
        else:
            hkl = hklkey(items)
            ndx = -1
            if hkl != None:
                for i,refl in enumerate(refls):
                    if refl.hkl == hkl:
                        ndx = i
                        printhkl()
                        break
                if ndx < 0:
                    newline('no such reflection')
        drawhkl()
        ableButtons(True)

    def margins( hor, ver):
        hmin = hor[0]
        hmax = hor[1]
        vmin = ver[0]
        vmax = ver[1]
        #newline('margins before %f %f %f %f'%(hmin,hmax,vmin,vmax))
        rh = hmax - hmin
        hmin = hmin - args.margin*rh
        hmax = hmax + args.margin*rh
        rv = vmax - vmin
        vmin = vmin - args.margin*rv
        vmax = vmax + args.margin*rv
        ax1.set_xlim(hmin,hmax)
        ax1.set_ylim(vmin,vmax)
        #newline('margins after %f %f %f %f'%(hmin,hmax,vmin,vmax))

    def IobsData():
        d = [ refl.iobs for refl in refls]
        x = numpy.array(d)
        limit = numpy.array( [numpy.amin(x), numpy.amax(x)] )
        ideal = limit
        return (x,limit,ideal,'iobs')

    def LogIobsData():
        d = [ refl.logiobs for refl in refls]
        x = numpy.array(d)
        limit = numpy.array( [0.0, numpy.amax(x)] )
        ideal = limit
        return (x,limit,ideal,'logiobs')

    def IoversigData():
        d = [ refl.ioversig for refl in refls]
        x = numpy.array(d)
        limit = numpy.array( [0.0, numpy.amax(x)] )
        ideal = limit
        return (x,limit,ideal,'ioversig')

    def SqrtIobsData():
        d = [ refl.sqrtiobs for refl in refls]
        x = numpy.array(d)
        limit = numpy.array( [0.0, numpy.amax(x)] )
        ideal = limit
        return (x,limit,ideal,'sqrtiobs')

    def IcalcData():
        d = [ refl.icalc for refl in refls]
        x = numpy.array(d)
        limit = numpy.array( [numpy.amin(x), numpy.amax(x)] )
        ideal = limit
        return (x,limit,ideal,'icalc')

    def LogIcalcData():
        d = [ refl.logicalc for refl in refls]
        x = numpy.array(d)
        limit = numpy.array( [0.0, numpy.amax(x)] )
        ideal = limit
        return (x,limit,ideal,'logicalc')

    def SqrtIcalcData():
        d = [ refl.sqrticalc for refl in refls]
        x = numpy.array(d)
        limit = numpy.array( [0.0, numpy.amax(x)] )
        ideal = limit
        return (x,limit,ideal,'sqrticalc')

    def SinthlData():
        d = [ refl.sinthl for refl in refls]
        x = numpy.array(d)
        limit = numpy.array([numpy.amin(x), numpy.amax(x) ])
        ideal = limit
        return (x,limit,ideal,'sinthl' )

    def Sinthl2Data():
        d = [ refl.sinthl2 for refl in refls]
        x = numpy.array(d)
        limit = numpy.array([numpy.amin(x), numpy.amax(x) ])
        ideal = limit
        return (x,limit,ideal,'sinthl2' )

    def SigData():
        d = [ refl.sigma for refl in refls]
        x = numpy.array(d)
        limit = numpy.array([numpy.amin(x), numpy.amax(x) ])
        ideal = limit
        return (x,limit,ideal,'sigma')

    def DelSigData(FullScale):
        d = [ refl.delsig for refl in refls]
        x = numpy.array(d)
        limit = numpy.array([numpy.amin(x), numpy.amax(x) ])
        if FullScale:
            a = max(abs(numpy.amin(limit)),abs(numpy.amax(limit)))
        else:
            a = 0.0
        ideal = numpy.array( [-a,a] )
        return (x,limit,ideal,'delta/sigma')
        
    def DelSig2Data():
        d = [ refl.delsig2 for refl in refls]
        x = numpy.array(d)
        limit = numpy.array([numpy.amin(x), numpy.amax(x) ])
        ideal = numpy.array( [0.0, 0.0] )
        return (x,limit,ideal,'delta/sigma^2')

    def Del2Sig2Data():
        d = [ refl.del2sig2 for refl in refls]
        x = numpy.array(d)
        limit = numpy.array([numpy.amin(x), numpy.amax(x) ])
        ideal = numpy.array( [0.0, 0.0] )
        return (x,limit,ideal,'delta^2/sigma^2')

    # def NormData( VerticalData ):
    #     yi = numpy.argsort( VerticalData[0] )
    #     h = gen117(nrefl)
    #     x = numpy.zeros( nrefl )
    #     for i in range( nrefl ):
    #         x[yi[i]] = h[i]
    #     limit = numpy.array([numpy.amin(x), numpy.amax(x) ])
    #     ideal = VerticalData[2]
    #     return (x,limit,ideal,'prob')

    def readSfacs():
        filename = os.path.join(os.path.dirname(sys.argv[0]),'sfac.tab')
        if not os.path.exists(filename):
            newline('sfactable file %s does not exist'%filename)
            return None
        sfacs={}
        f = open(filename,'r')
        for line in f:
            items = line.strip().split()
            sfacs[items[0]] = [ float(x) for x in items[1:] ]
        f.close()
        return sfacs

    def onefo(sfac,sinthl=0.0):
        a = [ sfac[i] for i in [0,2,4,6] ]
        b = [ sfac[i] for i in [1,3,5,7] ]
        c = sfac[8]
        fprime = sfac[9]
        fo = c + fprime
        for ai,bi in zip(a,b):
            fo += ai*numpy.exp(-bi*sinthl**2)
        return fo

    def calculate_fo2(sfacs,elementCount,sinthl=0.0):
        fo2 = 0.0
        for element in elementCount:
            fo2 += elementCount[element] * onefo(sfacs[element],sinthl=sinthl)**2
        return fo2

    def calculate_f000(sfacs,elementCount):
        f000 = 0.0
        for element in elementCount:
            f000 += elementCount[element] * onefo(sfacs[element])
        return f000

    def calculateIabs(bins):
        sfacs = readSfacs()
        if sfacs == None:
            sys.exit(1)
#assume bins are in sinthl2 = (sin(theta)/lambda)**2
        if args.verbose > 1:
            newline('calculateIabs')
            newline('  bins',bins)
        sinthl2s = []
        sinthl2s.append(bins[0]/2.0) 
        for i in range(1,len(bins)):
            sinthl2s.append( (bins[i]+bins[i-1])/2.0 )
        if args.verbose > 1:
            newline('  sinthl2s center of bins',sinthl2s)

        unit = re.split('(\d+)',args.unit)
        elementCount={}
        for (e,c) in zip(unit[0::2],unit[1::2]):
            elementCount[e.capitalize()] = args.z*int(c)
        newline('f000=%f'%calculate_f000(sfacs,elementCount))
        newline('     N','%9s'*11%('a(1)','b(1)','a(2)','b(2)','a(3)','b(3)','a(4)','b(4)','c',"f'","f''"))
        for elem in elementCount:
            nonewline('%2s %3d'%(elem, elementCount[elem]))
            newline('%9.4f'*len(sfacs[elem])%tuple(sfacs[elem]))

        iabs = []
        for sinthl2 in sinthl2s:
            sinthl=numpy.sqrt(sinthl2)
            fo2 = calculate_fo2(sfacs,elementCount,sinthl=sinthl)
            iabs.append(fo2)
        if args.verbose > 1:
            newline('  iabs',iabs)
        return iabs

    def BarData( hor, ver, func=None, name=None):
        if args.verbose > 1:
            newline('extracting bar data hor=%s ver=%s'%(hor,ver))
        d = [ getattr(refl,hor) for refl in refls]
        d = numpy.array(d)
        # bins contains the bin upper limits 
        # inds -> bin number
        bins,inds = GetBins(hor,args.nbin,d)
        # count (Sum(1.0) per bin)
        bc1 = numpy.bincount(inds)
        if ver == 'del2sig2':
            if args.verbose > 1:
                newline('before parcor',bc1)
            parcor = cifdict['lsparameters']/args.nbin
            bc1 = bc1-parcor
            if args.verbose > 1:
                newline('parcor=%d'%parcor)
                newline('after parcor',bc1)

        if args.verbose > 1:
            nonewline('%s bins:'%hor)
            newline(bins)
            nonewline('%s count:'%hor)
            newline(bc1)
            newline('total: %d'%sum(bc1))
        d = [ getattr(refl,ver) for refl in refls]
        values = numpy.array(d)
        # use values as weights for bincount (Sum(value) per bin)
        bc2 = numpy.bincount(inds,values)
        #normalize
        averages = numpy.nan_to_num(bc2/bc1)
        if func == 'iabs':
            iabs = calculateIabs(bins)
            if args.verbose > 1:
                newline('iabs',iabs)
                newline('before',averages)
            averages = averages/iabs
            if args.verbose > 1:
                newline('after',averages)
            averages = numpy.log(averages)
            if args.verbose > 1:
                newline('log',averages)

        elif func:
            averages = func(averages)
        if not name:
            name = ver
        return ( bins, averages, bc1, hor, name ) 

    def BarData2( hor ):
        if args.verbose > 1:
            newline('extracting bar data hor=',hor)
        d = [ getattr(refl,hor) for refl in refls]
        bins,inds = GetBins(hor,args.nbin,d)
        bc1 = numpy.bincount(inds)
        if args.verbose > 1:
            newline(hor,'bins=',bins)
            newline(hor,'count=',bc1)

        d2 = [ refl.iobs for refl in refls]
        values2 = numpy.array(d2)
        # calculate sum(Iobs) per bin
        bc2 = numpy.bincount(inds,values2)
        d3 = [ refl.icalc for refl in refls]
        values3 = numpy.array(d3)
        # calculate sum(Icalc) per bin
        bc3 = numpy.bincount(inds,values3)
        # calculate sum(iobs)/sum(icalc)
        bc4 = numpy.nan_to_num(bc2/bc3)
        # no need to normalize with bc1 (both numerator and denominator)
        return ( bins, bc4, bc1, hor, 'iobs/icalc' ) 

    def switchfunc(label):
        global mode
        global onscreen
        mode = modes.index(label)
        pf = plotfuncs[mode]
        ax1.cla()
        ax2.cla()
        #del bx.collections[:]
        if 'count' in onscreen:
            P1 = onscreen['count']
            P1.set_visible(False)
        plt.setp(ax2.get_yticklabels(), visible=False)
        onscreen = {}
        if args.verbose > 0 or True:
            newline('mode %d %s plot %s versus %s'%(mode,pf.type,pf.ytext,pf.xtext))
        if pf.type == 'scatter':
            ax1.set_zorder(1)
            ax2.set_zorder(0)
            ax3.set_zorder(0)
            if pf.Redline:
                L0,=ax1.plot(pf.horred,pf.verred,'r-')
                if pf.label:
                    ax1.text(0.99,0.01,
                             pf.label,
                             color='r',
                             horizontalalignment='right',
                             verticalalignment='bottom',
                             transform=ax1.transAxes)

                #print pf.horred,pf.verred
            L1,=ax1.plot(pf.hor,pf.ver,'b.',markersize=args.dotsize,picker=True,pickradius=5)
            L2,=ax1.plot(pf.hor[0],pf.ver[0],'r.',markersize=2*args.dotsize,visible=False)
            onscreen['hkl']=L2
            ax1.set_xlabel( alias(pf.xtext), size='xx-large')
            ax1.set_ylabel( alias(pf.ytext), size='xx-large')
            margins( pf.horlimit, pf.verlimit )
            drawhkl()
        elif pf.type == 'bar':
#            B0=ax1.bar(numpy.arange(pf.ver.size),pf.ver)
            B0=ax1.bar(numpy.arange(pf.ver.size),pf.ver,align='edge')
            if args.verbose > 0 or True:
                hormax = len(str(int(max(pf.hor))))
                vermax = len(str(int(max(pf.ver))))
                countmax = len(str(max(pf.count)))
                #newline(hormax,vermax,countmax)
                hormant = 2
                vermant = 2
                form =  '%%%ds %%%ds %%%ds'%(hormax+1,hormant+vermax+1,vermant+countmax)
                newline(form%('x','y','n'))
                form = '%%%d.%df %%%d.%df %%%dd'%(hormax+1+hormant,hormant,vermax+1+vermant,vermant,countmax)
                #newline(form)
                for h,v,c in zip(pf.hor,pf.ver,pf.count):
                    newline(form%(h,v,c))
                label = 'mean %.4f variance %.4f'%(numpy.mean(pf.ver),numpy.var(pf.ver))
                newline(label)
                ax1.text(0.0,1.00+0.3*args.charheight,
                         label,
                         color='r',
                         horizontalalignment='left',
                         verticalalignment='bottom',
                         transform=ax1.transAxes)

            ax1.set_xlabel( alias(pf.xtext), size='xx-large' )
            ax1.set_ylabel( alias(pf.ytext), size='xx-large' )
            margins( pf.horlimit, pf.verlimit )
            labels = []
            h = pf.hor.copy()
            h[-1] = h[-1] - EXTRAUPPER
            for f in h:
                labels.append('%.2f'%f)
            ax1.set_xticks(numpy.arange(pf.ver.size)+0.9)
            ax1.set_xticklabels(labels)
            plt.draw()
        elif pf.type == 'point':
            ax1.set_zorder(1)
            ax2.set_zorder(1)
            P0,=ax1.plot(pf.hor,pf.ver,'b.',marker='*',markersize=2*args.dotsize)
            if args.verbose > 0:
                for h,v,c in zip(pf.hor,pf.ver,pf.count):
                    newline('%.4f %.4f% 5d'%(h,v,c))
            ax1.set_xlabel( alias(pf.xtext), size='xx-large' )
            ax1.set_ylabel( alias(pf.ytext), size='xx-large' )
            margins( pf.horlimit, pf.verlimit )
#            labels = []
#            h = pf.hor.copy()
#            h[-1] = h[-1] - EXTRAUPPER
#            for f in h:
#                labels.append('%.2f'%f)
#            ax1.set_xticks(pf.hor)
#            ax1.set_xticklabels(labels)
            a,b = numpy.polyfit(pf.hor,pf.ver,1)
            ax1.plot(pf.hor, a*pf.hor + b, 'r')
            B = -0.5*a
            C = numpy.exp(b)
            k = 1.0/numpy.sqrt(C)
            label = 'y = %.3fx + %.3f\nB = %.2f    U = %.4f\nk = %.2f  osf = %.4f  C =  %.4f'%(a,b,B,B/(8.0*numpy.pi**2),k,numpy.sqrt(C),C)
            label = label.replace(' + -',' - ')
            newline(label)
            ax1.text(0.5,0.01,
                    label,
                    color='r',
                    horizontalalignment='center',
                    verticalalignment='bottom',
                    transform=ax1.transAxes)

            P1,=ax2.plot(pf.hor,pf.count,'c.',marker='x',markersize=2*args.dotsize)
            onscreen['count']=P1
            ax2.set_ylabel( 'count', size='xx-large' )
            ax2.set_ylim(0,1.1*max(pf.count))
            P1.set_visible(True)
            plt.setp(ax2.get_yticklabels(), visible=True)
            ax3.set_zorder(2)
            plt.draw()

    onscreen = {}

    parser = argparse.ArgumentParser(
        usage=_prog_usage, 
        description=_prog_description)
    parser.set_defaults(
        verbose= 0, 
        crit = -2.0,
        dotsize = 4,
        ignorefcfsigma = False,
        ignorestatus = False,
        lsparameters=None,
        margin = 0.01,
        mode = 0,
        nbin = 10,
        pagehor = 11.5*INCH, #29.7,
        pagever = 8.3*INCH, #21.0,
        savetype='png',
        weighta = -1.0,
        weightb = -1.0,
        unfold='',
        wavelength=None,
        wo='',
        wc='',
        z=1,
        )

    df = ' [default=%(default)s]'

    parser.add_argument(
        '-v','--verbose',
        action='count',
        help='more output')

    parser.add_argument(
        '-a','--weighta',
        metavar='f',
        type=float,
        help='shelx weight a')

    parser.add_argument(
        '-b','--weightb',
        metavar='f',
        type=float,
        help='shelx weight b')

    parser.add_argument(
        '--bins',
        nargs=10,
        metavar='f',
        type=float,
        help='10 upper resolution limits')

    parser.add_argument(
        '-c','--crit',
        type=float,
        metavar='f',
        help='minimum I/sig')

    parser.add_argument(
        '--cell',
        type=float,
        metavar=('a','b','c','alpha','beta','gamma'),
        nargs=6,
        help='6 cell parameters')

    parser.add_argument(
        '-d','--dotsize',
        type=int,
        metavar='i',
        help='dotsize'+df)

    parser.add_argument(
        '--ignorefcfsigma',
        action='store_true',
        help='ignore refined sigma in fcf file'+df)

    parser.add_argument(
        '--ignorestatus',
        action='store_true',
        help='ignore status flag in fco files'+df)

    parser.add_argument(
        '--lsparameters',
        metavar='i',
        type=int,
        help='nr of parameters in refinement [default from ciffile]')

    parser.add_argument(
        '-m','--mode',
        metavar='i',
        type=int,
        help='mode'+df)

    parser.add_argument(
        '--margin',
        metavar='f',
        type=float,
        help='margin (fraction of range) for axes'+df)

    parser.add_argument(
        '-n','--nbin',
        metavar='i',
        type=int,
        help='nr of bins'+df)

    parser.add_argument(
        '--pagehor',
        type=float,
        metavar='f',
        help='horizontal page size'+df)

    parser.add_argument(
        '--pagever',
        type=float,
        metavar='f',
        help='vertical page size'+df)

    parser.add_argument(
        '--savetype',
        metavar='filetype',
        help='set filetype for save'+df)

    parser.add_argument(
        '-u','--unit',
        metavar='content',
        help='unitcell contents: a list of atoms (f.e. Zn1C20H16)')

    parser.add_argument(
        '--wavelength',
        type=float,
        metavar='f',
        help='radiation wavelength (enables theta) [default from ciffile]')

    parser.add_argument(
        '--wo',
        help='write fobs to file')

    parser.add_argument(
        '--wc',
        help='write fcalc to file')

    parser.add_argument(
        '-z',
        type=int,
        metavar='i',
        help='cell content Z')


    parser.add_argument(
        'files',
        nargs='+',
        help='fcffilename')

    global args
    args = parser.parse_args()

    if args.verbose > 1:
        newline('args:',args)

    dmat = None
    rmat = None
    if args.cell:
        cell = list(args.cell)
        cell[3]=cell[3]*DEGREES
        cell[4]=cell[4]*DEGREES
        cell[5]=cell[5]*DEGREES
        dmat = daxdir(cell)
        printcell('cell from commandline',cell)
        if args.verbose > 0:
            newline(dmat)
        rmat = invert(dmat)
        if args.verbose > 1:
            newline(rmat)

    filelist = []
    abort = False
    for f in args.files:
        if os.path.exists(f):
            filelist.append(f)
        else:
            newline('unknown argument: ',f)
            abort=True

    if abort:
        raise UserWarning('unknown options')

    if len(filelist) != 1:
        raise UserWarning('need 1 filename')
    
    fullname = filelist[0]
    filename,filetype = os.path.splitext(fullname)

    cifdict,weightreport = SetWeightsAndWavelength(args,filename,filetype)
    if args.verbose > 0:
        newline('cifdict=',cifdict)

    rmat,inputrefls=ReadRefls(fullname,rmat=rmat,wavelength=cifdict['wavelength'])
    newline('  %s from %s'%(plural.reflections(inputrefls),fullname))

    if not args.ignorefcfsigma and hasattr(inputrefls[0],'refsig'):
        newline('ignoring',args.weightstatus)
        args.weightstatus = 'refined sigma from fcf file' 
        newline('using',args.weightstatus)
    else:
        newline('\n'.join(weightreport))
        
    dmat=invert(rmat)
    if args.verbose > 0:
        newline('dmat=',dmat)
    printcell('  cell',celldim(dmat.transpose()))
    if args.verbose > 0:
        newline('rmat=',rmat)
        printcell('  reciprocal cell',celldim(rmat))
        if args.verbose > 1:
            newline('a*=',matvec(rmat,numpy.array([1,0,0])))
            newline('b*=',matvec(rmat,numpy.array([0,1,0])))
            newline('c*=',matvec(rmat,numpy.array([0,0,1])))

    if args.wo:
        if os.path.exists(args.wo):
            newline('%s already exists'%args.wo)
        else:
            f = open(args.wo,'w')
            newline('created %s'%args.wo)
            form='%4d%4d%4d%8.2f%8.2f\n'
            for refl in inputrefls:
                f.write(form%(refl.h,refl.k,refl.l,refl.iobs,refl.sigma))
            f.write(form%(0,0,0,0.0,0.0))
            f.close()
    if args.wc:
        if hasattr(inputrefls[0],'icalc'):
            if os.path.exists(args.wc):
                newline('%s already exists'%args.wc)
            else:
                f = open(args.wc,'w')
                newline('created %s'%args.wc)
                form='%4d%4d%4d%8.2f%8.2f\n'
                for refl in inputrefls:
                    f.write(form%(refl.h,refl.k,refl.l,refl.icalc,refl.sigma))
                f.write(form%(0,0,0,0.0,0.0))
                f.close()
        else:
            newline('cannot create %s. no icalc available'%args.wc)
                
    refls = []
    nstatus = 0
    ncrit  = 0
    nweakiobs=0
    nweakicalc=0
    for refl in inputrefls:
        if filetype == '.fco' and not args.ignorestatus:
            if refl.status not in [0,'o']:
                nstatus +=1
                continue
        if refl.ioversig < args.crit:
            ncrit += 1
            continue
        if refl.iobs < 1.0:
            if args.verbose > 1:
                nonewline('too weak iobs')
                newline(refl)
            nweakiobs +=1
        if hasattr(refl,'icalc') and refl.icalc < 1.0:
            if args.verbose > 1:
                nonewline('too weak icalc')
                newline(refl)
            nweakicalc +=1
        refls.append(refl)
    nrefl = len(refls)
    newline('%s accepted'%(plural.reflections(nrefl)))
    if nstatus > 0:
        newline('  %s rejected on status'%(plural.reflections(nstatus)))
    if ncrit > 0:
        newline('  %s rejected with i/sig < %f'%(plural.reflections(ncrit),args.crit))
    if nweakiobs > 0:
        newline('  %s with iobs < 1.0 ==> log(iobs)=0 and sqrt(iobs)=1.0'%(plural.reflections(nweakiobs)))
    if nweakicalc > 0:
        newline('  %s with icalc < 1.0 ==> log(icalc)=0 and sqrt(icalc)=1.0'%(plural.reflections(nweakicalc)))
    if args.verbose > 1:
        if nweakiobs > 0 or nweakicalc > 0:
            newline('%s input'%plural.reflections(inputrefls))
            x = numpy.array( [ refl.sinthl for refl in inputrefls] )
            newline('sin(theta)/lambda input reflections from %f to %f'%(numpy.amin(x),numpy.amax(x)))
            newline('%s selected'%plural.reflections(refls))
            x = numpy.array( [ refl.sinthl for refl in refls] )
            newline('sin(theta)/lambda selected reflections from %f to %f'%(numpy.amin(x),numpy.amax(x)))

    if args.verbose > 1:
        scale = getScale()

    if args.verbose > 1:
        newline('setup plotfuncs')
    newline('Correcting Goof with %d refine parameters'%(cifdict['lsparameters']))

    plotfuncs = []
    if hasattr(refls[0],'icalc') and  hasattr(refls[0],'iobs'):
        pf = PLOTrecord()
        pf.scatter(IcalcData(), IobsData(), Redline=True, XY=True)
        plotfuncs.append(pf)

    if hasattr(refls[0],'logicalc') and  hasattr(refls[0],'logiobs'):
        pf = PLOTrecord()
        pf.scatter(LogIcalcData(), LogIobsData(), Redline=True, XY=True)
        plotfuncs.append(pf)

    if hasattr(refls[0],'sqrticalc') and  hasattr(refls[0],'sqrtiobs'):
        pf = PLOTrecord()
        pf.scatter(SqrtIcalcData(), SqrtIobsData(), Redline=True, XY=True)
        plotfuncs.append(pf)

    if hasattr(refls[0],'sigma') and  hasattr(refls[0],'iobs'):
        pf = PLOTrecord()
        pf.scatter(IobsData(),SigData())
        plotfuncs.append(pf)

    if hasattr(refls[0],'logiobs') and  hasattr(refls[0],'ioversig'):
        pf = PLOTrecord()
        pf.scatter(LogIobsData(), IoversigData(), Redline=False)
        plotfuncs.append(pf)

    if hasattr(refls[0],'logicalc') and  hasattr(refls[0],'delsig'):
        pf = PLOTrecord()
        pf.scatter(LogIcalcData(), DelSigData(False), Redline=True)
        plotfuncs.append(pf)

    if hasattr(refls[0],'delsig') and  hasattr(refls[0],'sinthl'):
        pf = PLOTrecord()
        pf.scatter(SinthlData(), DelSigData(False), Redline=True)
        plotfuncs.append(pf)

#    if hasattr(refls[0],'delsig2') and  hasattr(refls[0],'sinthl'):
#        pf = PLOTrecord()
#        pf.scatter(SinthlData(), DelSig2Data(), Redline=True)
#        plotfuncs.append(pf)

#    if hasattr(refls[0],'del2sig2') and  hasattr(refls[0],'sinthl'):
#        pf = PLOTrecord()
#        pf.scatter(SinthlData(), Del2Sig2Data(), Redline=True)
#        plotfuncs.append(pf)

    # if hasattr(refls[0],'delsig'):
    #     pf = PLOTrecord()
    #     pf.scatter(NormData( DelSigData(True)), DelSigData(True), Redline=True)
    #     plotfuncs.append(pf)

    if hasattr(refls[0],'delsig'):
        dsd = DelSigData(True)
        res = scipy.stats.probplot( dsd[0], fit=True, dist='norm')  # compute
        osm,osr = res[0]
        slope,intercept,r = res[1]
        newline('Normal probability (%s): slope=%.4f intercept=%.4f r=%.4f'%(plural.reflections(len(dsd[0])),slope,intercept,r))
        yi = numpy.argsort( dsd[0] )
        x = numpy.zeros( nrefl )
        for i in range( nrefl ):
            x[yi[i]] = osm[i]
        limitx = numpy.array([numpy.amin(x), numpy.amax(x) ])
        idealx = dsd[2]
        resultx = ( x,limitx,idealx,'prob')
        y = dsd[0]
        limity = dsd[1]
        #newline('idealx',idealx)
        idealy = numpy.array( [intercept + slope*idealx[0],intercept+slope*idealx[1]])
        #newline('idealy',idealy)
        resulty = ( y,limity,idealy,dsd[3])
        pf = PLOTrecord()
        label = 'Slope=%.4f Intercept=%.4f'%(slope,intercept)
        pf.scatter(resultx,resulty,Redline=True,label=label)
        plotfuncs.append(pf)

    if hasattr(refls[0],'iobs') and hasattr(refls[0],'sinthl2') and args.unit:
        pf = PLOTrecord()
        pf.point(BarData('sinthl2','iobs',func='iabs',name='lniobsiabs'))
        plotfuncs.append(pf)

    if hasattr(refls[0],'iobs') and hasattr(refls[0],'sinthl'):
        pf = PLOTrecord()
        pf.bar(BarData('sinthl','iobs'))
        plotfuncs.append(pf)

    if hasattr(refls[0],'del2sig2') and hasattr(refls[0],'sinthl'):
        pf = PLOTrecord()
        pf.bar(BarData('sinthl','del2sig2',func=numpy.sqrt,name='S'))
        plotfuncs.append(pf)

    if hasattr(refls[0],'del2sig2') and hasattr(refls[0],'icalc') :
        pf = PLOTrecord()
        pf.bar(BarData('icalc','del2sig2',func=numpy.sqrt,name='S'))
        plotfuncs.append(pf)

    if hasattr(refls[0],'iobs') and hasattr(refls[0],'icalc') and hasattr(refls[0],'sinthl') :
        pf = PLOTrecord()
        pf.bar(BarData2('sinthl'))
        plotfuncs.append(pf)

    if hasattr(refls[0],'iobs') and hasattr(refls[0],'icalc') :
        pf = PLOTrecord()
        #pf.bar(BarData2('logicalc'))
        pf.bar(BarData2('icalc'))
        plotfuncs.append(pf)

    if args.verbose > 0:
        newline(plural.nplural(plotfuncs,'plotfunc'))

    modes = []
    for pf in plotfuncs:
        if pf.type == 'scatter':
            c = '.'
        elif pf.type == 'bar':
            c = '|'
        elif pf.type == 'point':
            c = '*'
        modes.append( '%s %s $\mathit{vs}$ %s'%(c,alias(pf.ytext),alias(pf.xtext)) )

    if args.mode <= len(modes):
        mode = args.mode
    else:
        mode = 0
    rc('mathtext', default='regular')
# radio buttons with edges:
    if 'patch.force_edgecolor' in mpl.rcParams:
        mpl.rcParams['patch.force_edgecolor'] = True
    pagesize = (args.pagehor/INCH,args.pagever/INCH)
    fontsize=12.0
    args.charheight = fontsize/72.0/12
    args.charheight = 1.7*args.charheight
    aspect = args.pagehor/args.pagever
    fig=plt.figure(figsize=pagesize)
    fig.canvas.manager.set_window_title('anafcf')
    ax1 = fig.add_subplot(111)
    ax2 = ax1.twinx()
    #create ax3 on top of ax1. ax3 will log mouse movements 
    ax3 = ax1.figure.add_axes(ax1.get_position(True), sharex=ax1, sharey=ax1,
        frameon=False)
    ax3.xaxis.set_visible(False)
    ax3.yaxis.set_visible(False)
    margin=0.01
    left = 0.28
    right = 1.0-0.05-margin
    bottom = 0.09
    top = 1.0-margin*aspect-args.charheight
    if args.verbose > 0:
        newline('aspect=%f'%aspect)
        newline('left %f right %f bottom %f top %f'%(left,right,bottom,top))
    plt.subplots_adjust(left=left,right=right,bottom=bottom,top=top)

    ndx = -1

    # clickables from bottom up
    bottom,buttons = createButtons(0.01)
    bottom,radios = createRadios(bottom)

    # legenda's top down
    bottom = 0.99
    a = []
    a.append((fullname,'%d'%nrefl))
    if ncrit > 0:
        a.append(('crit','%.4f'%args.crit))
    if 'fcf' in args.weightstatus:
        a.append(('fcf sigma',''))
    else:
        a.append(('weighta','%.4f'%cifdict['weighta']))
        a.append(('weightb','%.4f'%cifdict['weightb']))
#    if wavelength > 0.01:
#        a.append(('lambda','%.5f'%cifdict['wavelength']))
    height = len(a)*args.charheight
    bottom = bottom - height
    CreateTable(bottom,height,a,columns=2)

    a = refls[0].legenda()
    height = len(a)*args.charheight
    bottom = bottom - GAP - height
    legendaa,legendab = CreateTable(bottom,height,a,columns=1)

    cid = fig.canvas.mpl_connect('pick_event', on_pick)
    fig.canvas.mpl_connect('key_press_event', pressrelease)
    #fig.canvas.mpl_connect('key_release_event', pressrelease)

    switchfunc(modes[mode])
    plt.show()
#    fname = 'pic.png'
#    newline('saving %s'%fname)
#    plt.savefig(fname,dpi=100,orientation='landscape',papertype='a4')

if __name__ == '__main__':
    main()
