"""Perform simple calculations with positions
"""
#
__version__='$Id: poscalc.py,v 1.4 2020/08/22 16:08:10 schreurs Exp $'
#
# (C) 1997--2004 Rob W.W. Hooft, Bruker AXS BV
#
from goniometer import position

def fromheader(h,fraction,hardware=None):
    """
    Calculate goniostat position at 'fraction' of the scan

    Will raise AttributeError if there is no scan info
    in the header.
    """
    if hardware is None:
        from hwdriver import dummydrv
        hardware=dummydrv.DummyGon(header=h)
    return h.p0.towards(h.p1,fraction).With(hardware=hardware)

def absdif(p0,p1):
    """
    Calculate "Rotational" difference between two positions in 'radians'.
    """
    import numpy
#    import numpy.linalg
    p0=p0.as_E()
    p1=p1.as_E()
    rotmat=numpy.dot(p0.goniometermatrix(),numpy.linalg.inv(p1.goniometermatrix()))
    trace=numpy.trace(rotmat)
    cosang=0.5*trace-0.5
    if cosang>=1.0:
       ang=0.0
    elif cosang<=-1.0:
       ang=numpy.pi
    else:
       ang=numpy.arccos(0.5*trace-0.5)
    return numpy.sqrt(0.0001*(p0.dx-p1.dx)**2+(p0.theta-p1.theta)**2+ang**2)

def equal(p0,p1,crit=1.e-4):
    """Are the two positions identical?
    """
    return absdif(p0,p1)<crit

    
