# using module re to pluralize most common english words
# (rule_tuple used as function default, so establish it first)
import re
# (pattern, search, replace) regex english plural rules tuple
rule_tuple = (
    ('[ml]ouse$', '([ml])ouse$', '\\1ice'),
    ('child$', 'child$', 'children'),
    ('booth$', 'booth$', 'booths'),
    ('foot$', 'foot$', 'feet'),
    ('ooth$', 'ooth$', 'eeth'),
    ('l[eo]af$', 'l([eo])af$', 'l\\1aves'),
    ('sis$', 'sis$', 'ses'),
    ('man$', 'man$', 'men'),
    ('ife$', 'ife$', 'ives'),
    ('eau$', 'eau$', 'eaux'),
    ('lf$', 'lf$', 'lves'),
    ('[sxz]$', '$', 'es'),
    ('[^aeioudgkprt]h$', '$', 'es'),
    ('(qu|[^aeiou])y$', 'y$', 'ies'),
    ('$', '$', 's')
    )

def regex_rules(rules=rule_tuple):
    for line in rules:
        pattern, search, replace = line
        yield lambda word: re.search(pattern, word) and re.sub(search, replace, word)

def plural(noun):
    for rule in regex_rules():
        result = rule(noun)
        if result:
            return result

def nplural(number,label):
    if hasattr(number, '__len__'):
        number = len(number)
    if abs(number) == 1:
        return '%d %s'%(number,label)
    else:
        return '%d %s'%(number,plural(label))

def testplural(number,label):
    if hasattr(number, '__len__'):
        number = len(number)
    if abs(number) == 1:
        return label
    else:
        return plural(label)

def singleplural(number,s,p):
    if hasattr(number, '__len__'):
        number = len(number)
    return (s if abs(number) == 1 else p)

def isare(n):
    return singleplural(n,'is','are')

def reflections(n):
    return nplural(n,'reflection')

def lines(n):
    return nplural(n,'line')

def records(n):
    return nplural(n,'record')
